import os
import importlib
from ._unauth import unauthorized


def get_commands_data(commands_dir=None) -> list:
    """
    This function is used to get commands data from a directory.

    It will import all python files in the given directory and get the command name and function from it.
    The function will be unauthorized if the author is not '@pamod_madubashana'.

    Args:
        commands_dir (str): The directory where the commands are located.

    Returns:
        list: A list of lists, where the first item in each list is the command name and the second item is the function.
    """
    if commands_dir ==None:return
    commands_data = []

    for file in os.listdir(commands_dir):
        if str(file).endswith('.py'):
            module_name = file[:-3] 
            if module_name == '__init__':continue
            if '/' in str(commands_dir):commands_dir = str(commands_dir).split('/')[-1]
            module_path = f'{commands_dir}.{module_name}'
            module = importlib.import_module(module_path)
            auther = getattr(module, '__author__', None)
            if auther != '@pamod_madubashana':
                function = unauthorized
            else:
                function_name = getattr(module, '__function__', None)
                function = getattr(module, function_name, None)
            command = getattr(module, '__command__', None)
            
            commands_data.append([command,function])
    print(', '.join(command[0] for command in commands_data) ," Ready")
    return commands_data


from pyrogram import Client as Application
from pyrogram import filters
from pyrogram.handlers import MessageHandler , CallbackQueryHandler , InlineQueryHandler , ChatJoinRequestHandler , ChatMemberUpdatedHandler, chat_join_request_handler , chat_member_updated_handler , user_status_handler,raw_update_handler,chosen_inline_result_handler,deleted_messages_handler,disconnect_handler,edited_message_handler,handler,inline_query_handler,poll_handler
from ._data_classess import Bot , Handlers , MessagesHandlers
from Easy_Bot.error import NoHandler
from ._commands import get_commands_data
from typing import Optional
from aiohttp import web
from ._web import web_server 
import asyncio
import os
from ._publicIp import get_public_ip , check_public_ip_reachable

LOGO = """
........................................
.#####...####...##...##...####...#####..
.##..##.##..##..###.###..##..##..##..##.
.#####..######..##.#.##..##..##..##..##.
.##.....##..##..##...##..##..##..##..##.
.##.....##..##..##...##...####...#####..
........................................    
   ├ ᴄᴏᴘʏʀɪɢʜᴛ © 𝟸𝟶𝟸𝟹-𝟸𝟶𝟸𝟺 ᴘᴀᴍᴏᴅ ᴍᴀᴅᴜʙᴀsʜᴀɴᴀ. ᴀʟʟ ʀɪɢʜᴛs ʀᴇsᴇʀᴠᴇᴅ.
   ├ ʟɪᴄᴇɴsᴇᴅ ᴜɴᴅᴇʀ ᴛʜᴇ  ɢᴘʟ-𝟹.𝟶 ʟɪᴄᴇɴsᴇ.
   └ ʏᴏᴜ ᴍᴀʏ ɴᴏᴛ ᴜsᴇ ᴛʜɪs ғɪʟᴇ ᴇxᴄᴇᴘᴛ ɪɴ ᴄᴏᴍᴘʟɪᴀɴᴄᴇ ᴡɪᴛʜ ᴛʜᴇ ʟɪᴄᴇɴsᴇ.
"""

def clear_terminal():
    if os.name == 'nt':
        os.system('cls')
    else: 
        os.system('clear')

class client(Application):
    def __init__(
            self,
            bot: Bot,
            start_function: callable,
    ):
        """
        Initialize a client instance.

        Args:
            bot (Bot): The bot class.
        """
        self.token = bot.token
        self.web = bot.web
        self.start_function = start_function
        super().__init__(
            name = bot.name,
            api_id = bot.api_id,
            api_hash = bot.api_hash,
            bot_token = bot.token,
            workers = bot.workers,
            in_memory = bot.in_memory
        )

    async def trigger_callback(self):
        await self.start_function()

    async def start(self):
        await super().start()
        self._bot = await self.get_me()
        clear_terminal()
        print(LOGO + "\n\n" )
        print(f"{self._bot.first_name} - @{self._bot.username} Started")

        if self.web:
            webapp = web.AppRunner(await web_server(self._bot))
            await webapp.setup()
            await web.TCPSite(webapp, "0.0.0.0", 8080).start()
            my_ip = get_public_ip()
            if str(self.web).startswith('http'):
                link = self.web
            elif await check_public_ip_reachable(my_ip):
                link = f"http://{my_ip}:8080"
            else:
                link = "locally http://0.0.0.0:8080"
            print(f"Webapp running on {link}")

        if self.start_function:
            await self.trigger_callback()

    async def stop(self, *args):
        clear_terminal()
        print(f"{self._bot.first_name} - @{self._bot.username} Stoped")
        await super().stop()

class pyroClient:
    def __init__(
            self,
            bot: Bot,
            handlers:Handlers = {}
    ):
        """
        Initialize a pyroClient instance.

        Args:
            bot (Bot): The bot class.
            handlers (HANDLERS, optional): Handlers for the bot. Defaults to {}.
        """
        print('Starting...')
        self.app = client(bot = bot ,start_function = handlers.start_function)
        
        if handlers == {} or handlers ==  None:
            raise NoHandler
        
        messages:MessagesHandlers = handlers.messages
        commands = handlers.commands
        callback = handlers.callback
        inline = handlers.inline
        join_request = handlers.join_request
        greeting = handlers.greeting
        self.start_function = handlers.start_function
        self.command_list = []

        if commands:
            commands_dir = commands
            commands = get_commands_data(commands_dir)
            
            for command , cmd_func in commands:
                self.app.add_handler(MessageHandler(cmd_func ,filters.command(command)))
                self.command_list.append(command)
                
        if messages:
            if messages.text:self.app.add_handler(MessageHandler(filters.text & ~filters.command(self.command_list),messages.text))
            if messages.poll:self.app.add_handler(MessageHandler(filters.poll,messages.poll))
            if messages.reply:self.app.add_handler(MessageHandler(filters.reply,messages.reply))
            if messages.audio:self.app.add_handler(MessageHandler(filters.audio,messages.audio))
            if messages.video:self.app.add_handler(MessageHandler(filters.video,messages.video))
            if messages.voice:self.app.add_handler(MessageHandler(filters.voice,messages.voice ))
            if messages.caption:self.app.add_handler(MessageHandler(filters.caption,messages.caption))
            if messages.contact:self.app.add_handler(MessageHandler(filters.contact,messages.contact))
            if messages.location:self.app.add_handler(MessageHandler(filters.location,messages.location))
            if messages.sticker:self.app.add_handler(MessageHandler(filters.sticker,messages.sticker))
            if messages.document:self.app.add_handler(MessageHandler(filters.document,messages.document))
            if messages.new_chat_photo:self.app.add_handler(MessageHandler(filters.new_chat_photo,messages.new_chat_photo))
            if messages.new_chat_title:self.app.add_handler(MessageHandler(filters.new_chat_title,messages.new_chat_title))
            if messages.new_chat_member:self.app.add_handler(MessageHandler(filters.new_chat_members,messages.new_chat_member))
            if messages.left_chat_memeber:self.app.add_handler(MessageHandler(filters.left_chat_member,messages.left_chat_memeber))
            if messages.pinned_message:self.app.add_handler(MessageHandler(filters.pinned_message,messages.pinned_message))
            if messages.all_status:self.app.add_handler(MessageHandler(filters.all,messages.all_status))

        if callback:
            self.app.add_handler(CallbackQueryHandler(callback))

        if inline:
            self.app.add_handler(InlineQueryHandler(inline))

        if join_request:
            self.app.add_handler(ChatJoinRequestHandler(join_request))

        if greeting:
            self.app.add_handler(ChatMemberUpdatedHandler(greeting))
    


        
    def stop(self):
        """
        Stop the bot. This method will stop the bot.
        """
        self.app.stop()



    def start(self):
        """
        Start the bot. This method will start the bot in either webhook or polling mode,
        depending on whether a webhook_url is provided.

        Args:
            drop_pending_updates (bool, optional): Whether to drop pending updates. Defaults to None.

        """
        try:
            self.app.run()
        except Exception as e:
            print(e)
            raise





import requests
import socket
import asyncio

def get_public_ip():
    try:
        response = requests.get('https://api.ipify.org?format=json')
        ip_data = response.json()
        return ip_data['ip']
    except requests.RequestException as e:
        print(f"Error retrieving IP address: {e}")
        return None

    
async def check_public_ip_reachable(ip: str, port: int = 8080):
    await asyncio.sleep(1)
    try:
        sock = socket.create_connection((ip, port), timeout=5)
        return True
    except (socket.timeout, socket.error):
        return False




from aiohttp import web
import psutil

routes = web.RouteTableDef()
Bot = None

@routes.get("/", allow_head=True)
async def root_route_handler(request):
    html_content = """
<!DOCTYPE html>
<html lang="en">
    <head>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Easy Bots</title>
    <style>
        * {
            margin: 0;
            padding: 0;
            box-sizing: border-box;
            font-family: 'Poppins', sans-serif;
            user-select: none;
        }
        
        body, html {
            height: 100%;
            width: 100%;
            font-family: Arial, sans-serif;
            overflow: hidden;
            display: flex;
            flex-direction: column;
            justify-content: center;
            align-items: center;
            color: #fff;
            background: linear-gradient(135deg, #ff7e5f, #4b280e, #192d4e, #174744);
            background-size: 400% 400%;
            animation: gradientBackground 20s ease infinite;
        }

        
        @keyframes gradientBackground {
            0% { background-position: 0% 50%; }
            50% { background-position: 100% 50%; }
            100% { background-position: 0% 50%; }
        }

        .big-text {
            position: absolute;
            top: 30px;
            font-size: 4em;
            font-weight: bold;
            text-shadow: 3px 3px 6px rgba(0, 0, 0, 0.3);
            z-index: 2;
            animation: pulse 10s infinite;
        }

        @keyframes pulse {
            0% { transform: scale(1); opacity: 1; }
            50% { transform: scale(1.1); opacity: 0.9; }
            100% { transform: scale(1); opacity: 1; }
        }
        .container{
            position: absolute;
            height: 400px;
            width: 800px;
            top: 30%;
            left: 50%;
            transform: translate(-50%);
            display: flex;
            justify-content: center;
            border-radius: 20px;
            background: #3a3939d2;
            backdrop-filter: blue(100px);
            z-index: 10;
        }
        #usage{
            display: flex;
            gap: 40px;
            color: white;
            transition: 0.5s;
        }

        #usage .circle{
            position: relative;
            top: 40px;
            left: 0px;
            width: 150px;
            height: 150px;
            display: flex;
            justify-content: center;
            align-items: center;
        }

        #usage .circle svg{
            position: relative;
            width: 150px;
            height: 150px;
            transform: rotate(90deg);
        }
        
        #usage .circle svg circle{
            width: 100%;
            height: 100%;
            fill:transparent;
            stroke: #19191979;
            stroke-width: 4;
            transform: translate(5px,5px);

        }
        #usage .circle svg circle:nth-child(2){
            stroke: var(--clr);
            stroke-dasharray: 440;
        }
        #usage div{
            position: absolute;
            text-align: center;
            font-weight: 500;

        }

        #usage .ap{
            position: relative;
            font-size: 1em;
            transform: translateX(-20px);
        }

        .dot{
            position: absolute;
            width: 100%;
            height: 100%;
            z-index: 10;
            display: flex;
            justify-content: center;
            transition: 0.1s;
        }

        .dot::before{
            content: '';
            bottom: -3px;
            position: absolute;
            width: 15px;
            height: 15px;
            background: var(--clr);
            border-radius: 50%;
            box-shadow: 0 0 20px var(--clr), 0 0 60px var(--clr);
        }

        #usage .info{
            position: absolute;
            bottom: -70px;
            font-size: 25px;
            color: #e6e6e6ab;
        }
        
        .container .line{
            position: relative;
            top: 280px;
            left:-650px;
            width: 150px;
            height: 1px;
            background: rgba(129, 129, 129, 0.534);
        }

        .container .hr1{
            position: relative;
            top: -90px;
            left: -360px;
            height: 750px;
            transform: rotate(90deg);
            border-color: rgba(160, 160, 160, 0.637);
        }
        .particles {
            position: absolute;
            width: 100%;
            height: 100%;
            overflow: hidden;
            z-index: 1;
            top: 0;
            left: 0;
        }
        .info-text{
            color: #e6e6e6e3;
            font-size: 25px;
        }
        .particle {
            position: absolute;
            display: block;
            width: 10px;
            height: 10px;
            background: rgba(255, 255, 255, 0.8);
            border-radius: 50%;
            opacity: 0.7;
            z-index: 0;
            animation: floatUp 10s infinite;
        }

        @keyframes floatUp {
            0% {
                transform: translateY(0) scale(1);
                opacity: 0.7;
            }
            50% {
                opacity: 1;
            }
            100% {
                transform: translateY(-1000px) scale(0.5);
                opacity: 0;
            }
        }
        .copyright{
            position: absolute;
            bottom: 0;
            right: 0;
            width: 120px;
            height: 50px;
            font-size: 20px;
            color: #a1a5a5d7;
            font-family: sans-serif;
            display: flex;
            justify-content: center;
            align-items: center;
            cursor: pointer;
            z-index: 10;
        }
        .copyright .logo a{
            z-index: 10;
            color: #a1a5a5d7;
            text-decoration: none;
            
        }
        .bot-details{
            position: absolute;
            bottom: 40px;
            left: 20px;
            color: #a1a5a5d7;
            
        }
        .bot-details .value-bot{
            position: absolute;
            left: 120px;
            color: rgb(196, 196, 196);
        }
        .bot-details .info-dot{
            position: absolute;
            left: 110px;
            color: #e2e2e2d7;
        }
        .bot-details .name{
            bottom: 20px;
            white-space: nowrap ;
        }
        .bot-details .username{
            bottom: 0px;
            white-space: nowrap ;
        }
        .total , .split , .current{
            position: absolute;
            top: 170px;
            color: #fdfdfdaf;
        }
        .total{
            left: 80px;
            font-size: 15px;
        }
        /* .split{
            position: absolute;

        } */
        .current{
            right: 80px;
            font-size: 15px;
        }
    </style>
</head>
<body>
    <div class="big-text">Easy Bots</div>
    <div class="container">
        <div id="usage">
            <div class="circle" style="--clr: #ff2972">
                <div class="dot cpu_dot"></div>
                <svg>
                    <circle cx="70" cy="70" r="70"></circle>
                    <circle cx="70" cy="70" r="70" id="cpu"></circle>
                </svg>
                <div class="info">CPU</div>
                <div class="info-text" id="cpu_usage">0%</div>
                <div class="total" id="cpuTotal">0 GHz</div>
                <div class="split">/</div>
                <div class="current" id="cpuCurrent">0 GHz</div>
            </div>
            <div class="circle" style="--clr: #fee800">
                <div class="dot ram_dot"></div>
                <svg>
                    <circle cx="70" cy="70" r="70"></circle>
                    <circle cx="70" cy="70" r="70" id="ram"></circle>
                </svg>
                <div class="info">RAM</div>
                <div class="info-text" id="ram_usage">0%</div>
                <div class="total" id="ramTotal">0 GB</div>
                <div class="split">/</div>
                <div class="current" id="ramCurrent">0 GB</div>

            </div>
            <div class="circle" style="--clr: #04fc43">
                <div class="dot disk_dot"></div>
                <svg>
                    <circle cx="70" cy="70" r="70"></circle>
                    <circle cx="70" cy="70" r="70" id="disk"></circle>
                </svg>
                <div class="info">DISK</div>
                <div class="info-text" id="disk_usage">0%</div>
                <div class="total" id="diskTotal">0 GB</div>
                <div class="split">/</div>
                <div class="current" id="diskCurrent">0 GB</div>

            </div>
            <div class="circle" style="--clr: #378cee">
                <div class="dot swap_dot"></div>
                <svg>
                    <circle cx="70" cy="70" r="70"></circle>
                    <circle cx="70" cy="70" r="70" id="swap"></circle>
                </svg>
                <div class="info">SWAP</div>
                <div class="info-text" id="swap_usage">0%</div>
                <div class="total" id="swapTotal">0 GB</div>
                <div class="split">/</div>
                <div class="current" id="swapCurrent">0 GB</div>
            </div>
        </div>
        <hr class="hr1">
        <div class="copyright">
            <div class="logo"><a href="https://t.me/Logic_lab_lk">Logic Lab</a></div>
        </div>
        <div class="bot-details">
            <div class="bot-name">
                <div class="info-bot">Bot Name</div>
                <div class="name info-dot">:</div>
                <div class="name value-bot" id="botname">Pamod ⚡</div>
            </div>
            <div class="bot-username">
                <div class="info-bot">Bot Username</div>
                <div class="username info-dot">:</div>
                <div class="username value-bot" id="botusername">pamod_bot</div>
            </div>
        </div>
    </div>
    <div class="particles">
        <div class="particle" style="top: 80%; left: 20%; animation-delay: 0s;"></div>
        <div class="particle" style="top: 90%; left: 40%; animation-delay: 2s;"></div>
        <div class="particle" style="top: 70%; left: 60%; animation-delay: 4s;"></div>
        <div class="particle" style="top: 85%; left: 80%; animation-delay: 6s;"></div>
        <div class="particle" style="top: 75%; left: 50%; animation-delay: 8s;"></div>
        <div class="particle" style="top: 65%; left: 30%; animation-delay: 10s;"></div>
        <div class="particle" style="top: 60%; left: 10%; animation-delay: 12s;"></div>
        <div class="particle" style="top: 55%; left: 70%; animation-delay: 14s;"></div>
        <div class="particle" style="top: 50%; left: 90%; animation-delay: 16s;"></div>
        <div class="particle" style="top: 45%; left: 15%; animation-delay: 18s;"></div>
        <div class="particle" style="top: 40%; left: 85%; animation-delay: 20s;"></div>
        <div class="particle" style="top: 35%; left: 60%; animation-delay: 22s;"></div>
        <div class="particle" style="top: 30%; left: 40%; animation-delay: 24s;"></div>
        <div class="particle" style="top: 25%; left: 20%; animation-delay: 26s;"></div>
        <div class="particle" style="top: 20%; left: 5%; animation-delay: 28s;"></div>
        <div class="particle" style="top: 15%; left: 75%; animation-delay: 30s;"></div>
        <div class="particle" style="top: 10%; left: 90%; animation-delay: 32s;"></div>
        <div class="particle" style="top: 5%; left: 50%; animation-delay: 34s;"></div>
        <div class="particle" style="top: 2%; left: 35%; animation-delay: 36s;"></div>
        <div class="particle" style="top: 3%; left: 10%; animation-delay: 38s;"></div>
    </div>

    <script>
        let cpu_usage = document.getElementById('cpu_usage');
        let ram_usage = document.getElementById('ram_usage');
        let disk_usage = document.getElementById('disk_usage');
        let swap_usage = document.getElementById('swap_usage');

        let cpuTotal = document.getElementById('cpuTotal');
        let ramTotal = document.getElementById('ramTotal');
        let diskTotal = document.getElementById('diskTotal');
        let swapTotal = document.getElementById('swapTotal');

        let cpuCurrent = document.getElementById('cpuCurrent');
        let ramCurrent = document.getElementById('ramCurrent');
        let diskCurrent = document.getElementById('diskCurrent');
        let swapCurrent = document.getElementById('swapCurrent');

        let botName = document.getElementById('botname');
        let botUsername = document.getElementById('botusername');

        let cpu = document.getElementById('cpu');
        let ram = document.getElementById('ram');
        let disk = document.getElementById('disk');
        let swap = document.getElementById('swap');

        let cpuDot = document.querySelector('.cpu_dot');
        let ramDot = document.querySelector('.ram_dot');
        let diskDot = document.querySelector('.disk_dot');
        let swapDot = document.querySelector('.swap_dot');

        let er = 0
        async function fetchUsage() {
            try {
                const response = await fetch('/usage');
                const data = await response.json();
                updateUsage(data.cpu, data.ram, data.disk, data.swap ,data.bot_name , data.bot_username ,data.cpu_current , data.ram_current ,data.disk_current ,data.swap_current , data.cpu_max , data.ram_toral , data.disk_total , data.swap_total);
            } catch (error) {
                if (er === 0){
                    console.log("Error fetching usage data:", error);
                    er = 1
                }
            }
        }

        function updateUsage(cpuUsage, ramUsage, diskUsage, swapUsage , bot_name , bot_username  ,cpu_current , ram_current , disk_current , swap_current ,cpu_max ,ram_toral , disk_total, swap_total ) {
            cpu_usage.innerHTML = cpuUsage + '%';
            ram_usage.innerHTML = ramUsage + '%';
            disk_usage.innerHTML = diskUsage + '%';
            swap_usage.innerHTML = swapUsage + '%'; 

            botName.innerHTML = bot_name
            botUsername.innerHTML = bot_username

            cpuTotal.innerHTML = cpu_max
            ramTotal.innerHTML = ram_toral
            diskTotal.innerHTML = disk_total
            swapTotal.innerHTML = swap_total

            cpuCurrent.innerHTML = cpu_current
            ramCurrent.innerHTML = ram_current
            diskCurrent.innerHTML = disk_current
            swapCurrent.innerHTML = swap_current

            cpu.style.strokeDashoffset = 440 - (cpuUsage * 440) / 100;
            ram.style.strokeDashoffset = 440 - (ramUsage * 440) / 100;
            disk.style.strokeDashoffset = 440 - (diskUsage * 440) / 100;
            swap.style.strokeDashoffset = 440 - (swapUsage * 440) / 100;

            cpuDot.style.transform = `rotate(${(cpuUsage * 360) / 100}deg)`;
            ramDot.style.transform = `rotate(${(ramUsage * 360) / 100}deg)`;
            diskDot.style.transform = `rotate(${(diskUsage * 360) / 100}deg)`;
            swapDot.style.transform = `rotate(${(swapUsage * 360) / 100}deg)`;

        }

    setInterval(fetchUsage, 1000);
    fetchUsage();


    </script>
</body>
</html>
"""
    return web.Response(text=html_content, content_type='text/html')


def mbgb(value): 
    value = value /1024 ** 2
    if int(value) > 1024:
        return str(round((value / 1024),2)) + " GB"
    else:
        return str(round(value,2)) + " MB"

@routes.get("/usage", allow_head=True)
async def usage_handler(request):
    global Bot
    cpu_usage = psutil.cpu_percent()
    ram_usage = psutil.virtual_memory().percent
    disk_usage = psutil.disk_usage('/').percent
    swap_usage = psutil.swap_memory().percent

    cpu_current = f"{(psutil.cpu_freq().current /1000):.2f} GHz" if psutil.cpu_freq() and psutil.cpu_freq().current else "Unknown"
    ram_current = mbgb(psutil.virtual_memory().cached)
    disk_current = mbgb(psutil.disk_usage('/').used)
    swap_current = mbgb(psutil.swap_memory().used)

    cpu_max =  f"{(psutil.cpu_freq().max /1000):.2f} GHz" if psutil.cpu_freq() and psutil.cpu_freq().max else "Unknown"
    memory_total= mbgb(psutil.virtual_memory().total)
    swap_total= mbgb(psutil.swap_memory().total)
    disk_total = mbgb(psutil.disk_usage('/').total)

    bot_name = Bot.first_name
    bot_username = Bot.username
    usage_data = {
        "cpu": cpu_usage,
        "ram": ram_usage,
        "disk": disk_usage,
        "swap": swap_usage,
        "bot_name": bot_name,
        "bot_username": bot_username,
        "cpu_current" : cpu_current,
        "ram_current" : ram_current,
        "disk_current" : disk_current,
        "swap_current" : swap_current,
        "cpu_max" : cpu_max,
        "ram_toral": memory_total,
        "disk_total": disk_total,
        "swap_total": swap_total,
    }
    return web.json_response(usage_data)

async def web_server(bot):
    global Bot
    Bot = bot
    web_app = web.Application(client_max_size=30000000)
    web_app.add_routes(routes)
    return web_app





class NoHandler(Exception):
    def __init__(self, message="No Handlers found , please set handlers to start bot"):
        self.message = message
        super().__init__(self.message)

class UnauthorizedCommandError(Exception):
    def __init__(self, message="Unauthorized command detected! You do not have permission to use this command."):
        self.message = message
        super().__init__(self.message)



class Bot:
    def __init__(
            self,
            name: str, 
            api_id: int, 
            api_hash: str, 
            token: str, 
            in_memory: bool, 
            workers: int=200,
            web : bool = False
            ):
        """
        Initialize a Bot instance.

        Args:
            name (str): The name of the bot.
            api_id (int): The API ID of the bot.
            api_hash (str): The API hash of the bot.
            token (str): The token of the bot.
            in_memory (bool): Whether to store the bot's data in memory.
            workers (int, optional): The number of workers to use for the bot. Defaults to 200.
        """
        self.name = name
        self.web = web
        self.api_id = api_id
        self.api_hash = api_hash
        self.token = token
        self.in_memory = in_memory
        self.workers = workers

class MessagesHandlers:
    def __init__(self,
        all : callable = None,
        bot_filter: callable = None,
        channel_chat_created: callable = None,
        text: callable = None,
        caption: callable = None,
        document: callable = None,
        audio: callable = None,
        video: callable = None,
        voice: callable = None,
        sticker: callable = None,
        contact: callable = None,
        location: callable = None,
        reply: callable = None,
        poll: callable = None,
        new_chat_member: callable = None,
        left_chat_memeber: callable = None,
        new_chat_title: callable = None,
        new_chat_photo: callable = None,
        pinned_message : callable = None,
        all_status : callable = None,
    ):
        """
        Initialize a MessagesHandlers instance.

        This class is used to store callback functions for the MessageHandler.

        Args:
            text: A callable that will be called when a text message is received.
            caption: A callable that will be called when a message with a caption is received.
            document: A callable that will be called when a document is received.
            audio: A callable that will be called when an audio is received.
            video: A callable that will be called when a video is received.
            voice: A callable that will be called when a voice message is received.
            sticker: A callable that will be called when a sticker is received.
            contact: A callable that will be called when a contact is received.
            location: A callable that will be called when a location is received.
            reply: A callable that will be called when a reply is received.
            poll: A callable that will be called when a poll is received.
            new_chat_member: A callable that will be called when a new chat member is received.
            left_chat_memeber: A callable that will be called when a chat member leaves the chat.
            new_chat_title: A callable that will be called when a chat title is changed.
            new_chat_photo: A callable that will be called when a chat photo is changed.
            message_auto_delete_timer_changed: A callable that will be called when the auto delete timer for a message is changed.
            pinned_message: A callable that will be called when a message is pinned.
            all_status: A callable that will be called when any of the above events are triggered.
        """
        self.all = all
        self.bot_filter = bot_filter
        self.channel_chat_created = channel_chat_created
        self.text = text
        self.caption = caption
        self.document = document
        self.audio = audio
        self.video = video
        self.voice = voice
        self.sticker = sticker
        self.contact = contact
        self.location = location
        self.reply = reply
        self.poll = poll
        self.new_chat_member = new_chat_member
        self.left_chat_memeber = left_chat_memeber
        self.new_chat_title = new_chat_title
        self.new_chat_photo = new_chat_photo
        self.pinned_message = pinned_message
        self.all_status = all_status


class Handlers:
    def __init__(self,
        start_function : callable = None,
        commands : str = None,
        messages : MessagesHandlers = None,
        callback : callable = None,
        inline : callable = None,
        join_request : callable = None,
        greeting : callable =None,
    ):
        """
        Initialize a HANDLERS instance.

        This class is used to store callback functions for the Bot.

        Args:
            start_function: A callable that will be called when the Bot is started.
            commands: A string with the path to the commands folder.
            messages: A MessagesHandlers instance with the callback functions for the MessageHandler.
            callback: A callable that will be called when a callback query is received.
            inline: A callable that will be called when an inline query is received.
            join_request: A callable that will be called when a join request is received.
            reaction: A callable that will be called when a message reaction is received.
            error: A callable that will be called when an error occurs.
        """
        self.commands = commands
        self.messages = messages
        self.callback = callback
        self.inline = inline
        self.join_request = join_request
        self.greeting = greeting
        self.start_function = start_function


from pyrogram import Client
from pyrogram.types import Message 
from pyrogram.enums import ParseMode
from Easy_Bot.error import UnauthorizedCommandError


async def unauthorized(client:Client, message:Message):
    text =  "<b>Unauthorized command detected! </b>\n\nYou do not have permission to use this command."
    await message.reply_text(text=text,parse_mode=ParseMode.HTML)
    raise UnauthorizedCommandError